# This file was automatically generated Sat May 26 09:02:10 CDT 2007
# by java2perl6 0.1 from decompiling IntTest.class using command line flags:
#   --classpath testjavas

role IntTest {
    method array_returner(
        Str v1,
        Int v2,
        Array of Num v3,
        Array of Array of Num v4
    ) returns Array of Array of Int { ... }

    multi method object_returner() returns Str { ... }

    multi method object_returner(String v1) returns Str { ... }
}
